==> houses_and_lockersV2.1.tar.gz: houses/house_exit.c <==

 /* House_exit.c   V2.1      Metaprat       6th April 1992  */

 /* This object holds the door state transition table.  It  */
 /* does *not* do any actual door operations, or deal with  */
 /* any house data.  Its loaded, and the single reference   */
 /* copy used to look up the results of door state changes. */


#include "house_defines.h"


string state_transition_table;


id(str) {
   return str == "house exit";
}


short() {
   return "house exit object";
}


reset(arg) {
   if (arg) {
      return;
   }
   /*   status           type               actions             */
   /*     (unknown)        erroneous          --                */
   /*     DOOR_NONE        no exit            --                */
   /*     DOOR_NORM        normal exit        enter             */
   /*     DOOR_L_LOCK      lockable doors     unlock            */
   /*     DOOR_L_UNLK        "        "       lock, open        */
   /*     DOOR_L_OPEN        "        "       enter, close      */
   /*     DOOR_O_CLOS      openable doors     open              */
   /*     DOOR_O_OPEN         "       "       enter, close      */
   /*     DOOR_PORTAL      owner only doors   (enter if owner)  */
   /*     DOOR_ONEWAY      leads to outside the house           */
   /*     DOOR_HIDDEN      not listed in ob exits msg, nor map  */
   /*                                                           */
   /* table syntax:                                             */
   /*                                                           */
   /* ({  [({ door_status [,"property", "action" ]* })]* })     */
   /* where:                                                    */
   /*   door_stat is the current status of the exit             */
   /*   property  is some possible manipulation,                */
   /*   action    is the result of that manipulation, in the    */
   /*      format:                                              */
   /*        %d       - success, new_door_status = n            */
   /*        %s       - no change, string written               */
   /*        O:%d:%s  - if owner, success & new_status = n,     */
   /*                   else no change and string written.      */
   /*        W:%d:%s  - if wizard, success & new_status = %d.   */
   /*                   else no change and string written.      */

   state_transition_table = ({
({ -9999,      /* unrecognised door status */
   "exam",  "The door looks a bit strange, for some reason.\n",
   "enter", "You fail to get through the door.  Perhaps something is wrong.\n",
   "open",  "You can't open the door.  Perhaps something is wrong.\n",
   "close", "You can't close the door.  Perhaps something is wrong.\n",
   "unlock","You can't unlock the door.  Perhaps something is wrong.\n",
   "lock",  "You can't lock the door.  Perhaps something is wrong.\n",
   "knock", "You can't knock at the door.  Perhaps something is wrong.\n"}),
({ DOOR_NONE,  /* valid status, but door doesn't exist */
   "exam",  "There is no (obvious) exit in that direction.\n",
   "enter", "Eh?  There isn't an exit there..\n",
   "open",  "Eh?  How can you open a door which doesn't exist?\n",
   "close", "Eh?  How can you close a door which doesn't exist?\n",
   "unlock","Eh?  How can you unlock a door which doesn't exist?\n",
   "lock",  "Eh?  How can you lock a door which doesn't exist?\n",
   "knock", "Eh?  How can you knock at a door which doesn't exist?\n"}),
({ DOOR_NORM,
   "exam",  "Just a normal exit, from the looks of it.\n",
   "enter", DOOR_NORM,
   "open",  "The exit isn't a door, its always possible to walk though it.\n",
   "close", "Thats just an egress.. not even a door as such.\n",
   "unlock","Thats just an exit, its always open.\n",
   "lock",  "Thats just an exit, it isn't a door as such.\n",
   "knock", "There's nothing to knock on, just a large gap.\n"}),
({ DOOR_L_LOCK,
   "exam",  "The door is locked at the moment.\n",
   "enter", "Sorry, the door is locked.\n",
   "open",  "But the door is locked!  You have to unlock it first.\n",
   "close", "But the door is locked!  Therefore it is already closed.\n",
   "unlock","O:" + DOOR_L_UNLK + ":You don't have the key.\n",
   "lock",  "Its already locked!\n",
   "knock", DOOR_L_LOCK}),
({ DOOR_L_UNLK,
   "exam",  "The door is unlocked, but closed.\n",
   "enter", "Sorry, the door is closed.\n",
   "open",  DOOR_L_OPEN,
   "close", "Its already closed!\n",
   "unlock","Its already unlocked.\n",
   "lock",  "O:" + DOOR_L_LOCK + ":You don't have the key.\n",
   "knock", DOOR_L_UNLK}),
({ DOOR_L_OPEN,
   "exam",  "The door is unlocked, and open.\n",
   "enter", DOOR_L_OPEN,
   "open",  "Its already open.\n",
   "close", DOOR_L_UNLK,
   "unlock","Its already both unlocked and open.n",
   "lock",  "You have to close it first.\n",
   "knock", DOOR_L_OPEN}),
({ DOOR_O_CLOS,
   "exam",  "The door is closed.\n",
   "enter", "Sorry, the door is closed.\n",
   "open",  DOOR_O_OPEN,
   "close", "Its already closed!\n",
   "unlock","There is no lock on the door.\n",
   "lock",  "There is no lock on the door.\n",
   "knock", DOOR_O_CLOS}),
({ DOOR_O_OPEN,
   "exam",  "The door is open.\n",
   "enter", DOOR_O_OPEN,
   "open",  "Its already open.\n",
   "close", DOOR_O_CLOS,
   "unlock","There is no lock on the door, and its open anyway.\n",
   "lock",  "There is no lock on the door, and its open anyway.\n",
   "knock", DOOR_O_OPEN}),
({ DOOR_PORTAL,
   "exam",  "A strange flickering field of light obscures the exit.\n",
   "enter", "O:" + DOOR_PORTAL + ":You bounce off the field of white fire.\n",
   "open",  "How do you propose to open a field of mystic fire?? ;)\n",
   "close", "How do you propose to close a field of mystic fire?? ;)\n",
   "unlock","How do you propose to unlock a field of mystic fire?? ;)\n",
   "lock",  "How do you propose to lock a field of mystic fire?? ;)\n",
   "knock", "The white fire burns your fingers.\n"}),
({ DOOR_ONEWAY,
   "exam",  "This leads out of the house.\n",
   "enter", DOOR_ONEWAY,
   "open",  "Its always open.\n",
   "close", "You can't close this type of exit.\n",
   "unlock","Its always open, so is by definition unlocked.\n",
   "lock",  "This exit is impossible to lock.\n",
   "knock", "You can't knock on it.\n"}),
({ DOOR_HIDDEN,
   "exam",  "Looking closer, you notice a secret exit!\n",
   "enter", DOOR_HIDDEN,
   "open",  "The exit isn't a door, its always possible to walk though it.\n",
   "close", "Thats just an egress.. not even a door as such.\n",
   "unlock","Thats just an exit, its always open.\n",
   "lock",  "Thats just an exit, it isn't a door as such.\n",
   "knock", "There's nothing to knock on, just a large gap.\n"}) });
}



get_status_property_action(door_status, property) {
   int counter;
   int status_entry;

   /* Get the subarray from the manipulation table for the appropriate */
   /* door_status.  If nothing is found, then use the first subarray.  */
   status_entry = 0;
   for (counter = 1; counter < sizeof(state_transition_table); counter ++) {
      if (state_transition_table[counter][0] == door_status) {
         status_entry = counter;
         break;
      }
   }
   /* now search for the matching property within the table, and return */
   /* the action string if it's found.                                  */
   for (counter = 1;
        counter < sizeof(state_transition_table[status_entry]);
        counter += 2) {
      if (state_transition_table[status_entry][counter] == property) {
         return state_transition_table[status_entry][counter + 1];
      }
   }
   return "ERROR! get_status_property_action failed to locate action for " +
          "status: " + door_status + ", property = " + property + ".\n";
}




manipulate_door(main_hall, indx, which_door) {
   int    door_status;
   string fail_msg;
   string action;
   int    action_succeeded;
   int    new_status;

   door_status = main_hall->get_exit_status(indx, which_door);

   /* The 'action' returned is in one of the following formats: */
   /*   %d      - new door status (operation 'succeeds')        */
   /*   %s      - failure msg to user (operation 'fails')       */
   /*   O:%d:%s - New status if owner, else write failure msg   */
   /*   W:%d:%s - New status if wizard, else write fail msg     */

   action = get_status_property_action(door_status, query_verb());

   /* Process the action, and decide if the manipulation failed or */
   /* succeeded.                                                   */
   if (stringp(action) && (action)) {
      if (sscanf(action, "O:%d:%s", new_status, fail_msg) == 2) {
         action_succeeded = (GETOWNER == MYNAME);
      }
      else if (sscanf(action, "W:%d:%s", new_status, fail_msg) == 2) {
         action_succeeded = (IAMWIZ);
      }
      else {
         action_succeeded = 0;
         fail_msg = action;
      }
   }
   else if (intp(action)) {
      action_succeeded = 1;
      new_status = action;
   }

   /* perform the action if it succeeded, or write the fail msg otherwise. */
   if (action_succeeded) {
      if (new_status != door_status) {
         main_hall->set_exit_status(indx, which_door, new_status);
      }
      main_hall->notify_room(indx,
         MYCNAME + " " + query_verb() + "s the " +
         which_door + " door.\n");
      main_hall->notify_room(main_hall->direction_room(indx, which_door),
         "Someone "+query_verb()+"s the " +
         main_hall->opposite_direction(which_door) +
         " door, from the other side.\n");
   }
   else {
      write(fail_msg);
   }
}

==> houses_and_lockersV2.2.tar.gz: houses/house_exit.c <==

 /* House_exit.c   V2.1      Metaprat       6th April 1992  */

 /* This object holds the door state transition table.  It  */
 /* does *not* do any actual door operations, or deal with  */
 /* any house data.  Its loaded, and the single reference   */
 /* copy used to look up the results of door state changes. */


#include "house_defines.h"
#include "array_funcs.c"

string state_transition_table;


reset(arg) {
   if (arg) {
      return;
   }
   /*   status           type               actions             */
   /*     (unknown)        erroneous          --                */
   /*     DOOR_NONE        no exit            --                */
   /*     DOOR_NORM        normal exit        enter             */
   /*     DOOR_L_LOCK      lockable doors     unlock            */
   /*     DOOR_L_UNLK        "        "       lock, open        */
   /*     DOOR_L_OPEN        "        "       enter, close      */
   /*     DOOR_O_CLOS      openable doors     open              */
   /*     DOOR_O_OPEN         "       "       enter, close      */
   /*     DOOR_PORTAL      owner only doors   (enter if owner)  */
   /*     DOOR_ONEWAY      leads to outside the house           */
   /*     DOOR_HIDDEN      not listed in ob exits msg, nor map  */

   /*   door_stat is the current status of the exit             */
   /*   property  is some possible manipulation,                */
   /*   action    is the result of that manipulation, in the    */
   /*      format:                                              */
   /*        %d       - success, new_door_status = n            */
   /*        %s       - no change, string written               */
   /*        O:%d:%s  - if owner, success & new_status = n,     */
   /*                   else no change and string written.      */
   /*        W:%d:%s  - if wizard, success & new_status = %d.   */
   /*                   else no change and string written.      */

   state_transition_table = ({
-9999,      /* unrecognised door status */
({ "exam",  "The door looks a bit strange, for some reason.\n",
   "enter", "You fail to get through the door.  Perhaps something is wrong.\n",
   "open",  "You can't open the door.  Perhaps something is wrong.\n",
   "close", "You can't close the door.  Perhaps something is wrong.\n",
   "unlock","You can't unlock the door.  Perhaps something is wrong.\n",
   "lock",  "You can't lock the door.  Perhaps something is wrong.\n",
   "knock", "You can't knock at the door.  Perhaps something is wrong.\n"}),
DOOR_NONE,  /* valid status, but door doesn't exist */
({ "exam",  "There is no (obvious) exit in that direction.\n",
   "enter", "Eh?  There isn't an exit there..\n",
   "open",  "Eh?  How can you open a door which doesn't exist?\n",
   "close", "Eh?  How can you close a door which doesn't exist?\n",
   "unlock","Eh?  How can you unlock a door which doesn't exist?\n",
   "lock",  "Eh?  How can you lock a door which doesn't exist?\n",
   "knock", "Eh?  How can you knock at a door which doesn't exist?\n"}),
DOOR_NORM,
({ "exam",  "Just a normal exit, from the looks of it.\n",
   "enter", DOOR_NORM,
   "open",  "The exit isn't a door, its always possible to walk though it.\n",
   "close", "Thats just an egress.. not even a door as such.\n",
   "unlock","Thats just an exit, its always open.\n",
   "lock",  "Thats just an exit, it isn't a door as such.\n",
   "knock", "There's nothing to knock on, just a large gap.\n"}),
DOOR_L_LOCK,
({ "exam",  "The door is locked at the moment.\n",
   "enter", "Sorry, the door is locked.\n",
   "open",  "But the door is locked!  You have to unlock it first.\n",
   "close", "But the door is locked!  Therefore it is already closed.\n",
   "unlock","O:" + DOOR_L_UNLK + ":You don't have the key.\n",
   "lock",  "Its already locked!\n",
   "knock", DOOR_L_LOCK}),
DOOR_L_UNLK,
({ "exam",  "The door is unlocked, but closed.\n",
   "enter", "Sorry, the door is closed.\n",
   "open",  DOOR_L_OPEN,
   "close", "Its already closed!\n",
   "unlock","Its already unlocked.\n",
   "lock",  "O:" + DOOR_L_LOCK + ":You don't have the key.\n",
   "knock", DOOR_L_UNLK}),
DOOR_L_OPEN,
({ "exam",  "The door is unlocked, and open.\n",
   "enter", DOOR_L_OPEN,
   "open",  "Its already open.\n",
   "close", DOOR_L_UNLK,
   "unlock","Its already both unlocked and open.n",
   "lock",  "You have to close it first.\n",
   "knock", DOOR_L_OPEN}),
DOOR_O_CLOS,
({ "exam",  "The door is closed.\n",
   "enter", "Sorry, the door is closed.\n",
   "open",  DOOR_O_OPEN,
   "close", "Its already closed!\n",
   "unlock","There is no lock on the door.\n",
   "lock",  "There is no lock on the door.\n",
   "knock", DOOR_O_CLOS}),
DOOR_O_OPEN,
({ "exam",  "The door is open.\n",
   "enter", DOOR_O_OPEN,
   "open",  "Its already open.\n",
   "close", DOOR_O_CLOS,
   "unlock","There is no lock on the door, and its open anyway.\n",
   "lock",  "There is no lock on the door, and its open anyway.\n",
   "knock", DOOR_O_OPEN}),
DOOR_PORTAL,
({ "exam",  "A strange flickering field of light obscures the exit.\n",
   "enter", "O:" + DOOR_PORTAL + ":You bounce off the field of white fire.\n",
   "open",  "How do you propose to open a field of mystic fire?? ;)\n",
   "close", "How do you propose to close a field of mystic fire?? ;)\n",
   "unlock","How do you propose to unlock a field of mystic fire?? ;)\n",
   "lock",  "How do you propose to lock a field of mystic fire?? ;)\n",
   "knock", "The white fire burns your fingers.\n"}),
DOOR_ONEWAY,
({ "exam",  "This leads out of the house.\n",
   "enter", DOOR_ONEWAY,
   "open",  "Its always open.\n",
   "close", "You can't close this type of exit.\n",
   "unlock","Its always open, so is by definition unlocked.\n",
   "lock",  "This exit is impossible to lock.\n",
   "knock", "You can't knock on it.\n"}),
DOOR_HIDDEN,
({ "exam",  "Looking closer, you notice a secret exit!\n",
   "enter", DOOR_HIDDEN,
   "open",  "The exit isn't a door, its always possible to walk though it.\n",
   "close", "Thats just an egress.. not even a door as such.\n",
   "unlock","Thats just an exit, its always open.\n",
   "lock",  "Thats just an exit, it isn't a door as such.\n",
   "knock", "There's nothing to knock on, just a large gap.\n"}) });
}



get_status_property_action(door_status, property) {
   string sub_table;

   /* Get the subarray from the manipulation table for the appropriate */
   /* door_status.  If nothing is found, then use the first subarray.  */
   sub_table = search_paired_array(state_transition_table, door_status);
   if (!sub_table) {
      sub_table = state_transition_table[1];
   }

   /* now search for the matching property within the table, and return */
   /* the action string if it's found.                                  */
   return search_paired_array(sub_table, property);
}




manipulate_door(main_hall, indx, which_door) {
   int    door_status;
   string fail_msg;
   string action;
   int    action_succeeded;
   int    new_status;

   door_status = main_hall->get_exit_status(indx, which_door);

   /* The 'action' returned is in one of the following formats: */
   /*   %d      - new door status (operation 'succeeds')        */
   /*   %s      - failure msg to user (operation 'fails')       */
   /*   O:%d:%s - New status if owner, else write failure msg   */
   /*   W:%d:%s - New status if wizard, else write fail msg     */

   action = get_status_property_action(door_status, query_verb());

   /* Process the action, and decide if the manipulation failed or */
   /* succeeded.                                                   */
   if (stringp(action) && (action)) {
      if (sscanf(action, "O:%d:%s", new_status, fail_msg) == 2) {
         action_succeeded = (GETOWNER == MYNAME);
      }
      else if (sscanf(action, "W:%d:%s", new_status, fail_msg) == 2) {
         action_succeeded = (IAMWIZ);
      }
      else {
         action_succeeded = 0;
         fail_msg = action;
      }
   }
   else if (intp(action)) {
      action_succeeded = 1;
      new_status = action;
   }

   /* perform the action if it succeeded, or write the fail msg otherwise. */
   if (action_succeeded) {
      if (new_status != door_status) {
         main_hall->set_exit_status(indx, which_door, new_status);
      }
      main_hall->notify_room(indx,
         MYCNAME + " " + query_verb() + "s the " +
         which_door + " door.\n");
      main_hall->notify_room(main_hall->direction_room(indx, which_door),
         "Someone "+query_verb()+"s the " +
         main_hall->opposite_direction(which_door) +
         " door, from the other side.\n");
   }
   else {
      write(fail_msg);
   }
}
